/*
 * Copyright (c) 2009 Pervasive Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the license.
 *
 * $Id: $
 */

package org.ogce.xregistry.security;

import java.io.InputStream;
import java.security.cert.X509Certificate;
import java.util.Properties;

import org.apache.log4j.Logger;
import org.globus.myproxy.MyProxy;
import org.ietf.jgss.GSSCredential;
import org.ogce.xregistry.client.XRegistryClient;
import org.ogce.xregistry.utils.XRegClientConstants;
import org.ogce.xregistry.utils.XRegistryClientException;

/**
 * @author Raminderjeet Singh
 */
public class ApplicationProperties {

	/**
	 * 
	 */
	private static final long serialVersionUID = -2270092298283648553L;
	private XRegistryClient registryClient;
	private Properties properties;
	private String xRegistryUrl = null;
	protected GSSCredential gssCredential;
	private String userDN;
	private MyProxyCredentials credentials;
	private static final Logger log = Logger.getLogger(ApplicationProperties.class);

	/**
	 * 
	 * Constructs a ApplicationGlobalContext.
	 * 
	 * @throws GfacGUIException
	 */

	public ApplicationProperties(String propertyfile) throws XRegistryClientException {
		log.setLevel(org.apache.log4j.Level.INFO);
		loadConfigration(propertyfile);
		gssCredential = credentials.getGssCredential();
	}

	/**
	 * Provide hostcert key and trusted CA's location to create GSSCredential
	 * @param hostcertsKeyFile
	 * @param trustedcertsFile
	 * @throws Exception
	 */
	public ApplicationProperties(String hostcertsKeyFile, String trustedcertsFile) throws XRegistryClientException {
		log.setLevel(org.apache.log4j.Level.INFO);
		if (gssCredential == null) {
			if (credentials == null) {
				this.credentials = new MyProxyCredentials();
			}
			gssCredential = credentials.getGssCredential(hostcertsKeyFile,trustedcertsFile);
		}

	}
	/**
	 * 
	 * @param myproxyServer
	 * @param myproxyPort
	 * @param myproxyUsername
	 * @param myproxyPassphrase
	 * @param myproxyLifetime
	 * @param trustedcertsFile
	 * @throws Exception
	 */
	public ApplicationProperties(String myproxyServer, int myproxyPort, String myproxyUsername, String myproxyPassphrase,
			int myproxyLifetime,String trustedcertsFile) throws XRegistryClientException {
		log.setLevel(org.apache.log4j.Level.INFO);
		if (gssCredential == null) {
			if (credentials == null) {
				this.credentials = new MyProxyCredentials(myproxyServer, myproxyPort, myproxyUsername,  myproxyPassphrase,
						myproxyLifetime,trustedcertsFile);
			}
			gssCredential = credentials.getGssCredential();
		}
	}
	/**
	 * 
	 * @param trustedCertsFile
	 * @param isdirectory
	 */
    public static X509Certificate[] loadCeritificates(String trustedCertsFile) {
    	MyProxyCredentials credentials = new MyProxyCredentials();
		credentials.loadCertificate(trustedCertsFile);
        return credentials.getTrustedCertificates();
    }
    /**
	 * 
	 * @param propertyfile
	 * @throws Exception
	 */
	private void loadConfigration(String propertyfile) throws XRegistryClientException {
		try {
			if (properties == null) {
				ClassLoader classLoader = Thread.currentThread().getContextClassLoader();
				InputStream propertyStream = classLoader.getResourceAsStream(propertyfile);
				properties = new Properties();
				if (credentials == null) {
					this.credentials = new MyProxyCredentials();
				}
				if (propertyStream != null) {
					properties.load(propertyStream);
					String xRegistryUrlTmp = properties.getProperty(XRegClientConstants.REGISTRYURL);
					if (xRegistryUrlTmp != null) {
						this.xRegistryUrl = xRegistryUrlTmp;
					}
					String myproxyServerTmp = properties.getProperty(XRegClientConstants.MYPROXYSERVER);
					if (myproxyServerTmp != null) {
						this.credentials.setMyproxyHostname(myproxyServerTmp.trim());
					}
					String myproxyPortTmp = properties.getProperty(XRegClientConstants.MYPROXYSERVERPORT);
					if (myproxyPortTmp != null && myproxyPortTmp.trim().length() > 0) {
						this.credentials.setMyproxyPortNumber(Integer.parseInt(myproxyPortTmp.trim()));
					} else {
						this.credentials.setMyproxyPortNumber(MyProxy.DEFAULT_PORT);
					}
					String myproxyUserTmp = properties.getProperty(XRegClientConstants.MYPROXYUSER);
					if (myproxyUserTmp != null) {
						this.credentials.setMyproxyUserName(myproxyUserTmp.trim());
					}
					String myproxyPassTmp = properties.getProperty(XRegClientConstants.MYPROXYPASS);
					if (myproxyPassTmp != null) {
						this.credentials.setMyproxyPassword(myproxyPassTmp.trim());
					}
					this.credentials.setHostcertsKeyFile(properties.getProperty(XRegClientConstants.SSL_HOSTCERTSKEY));
					this.credentials.setTrustedCertsFile(properties.getProperty(XRegClientConstants.SSL_TRUSTEDCERTS));

				}
			}

		} catch (Exception e) {
			throw new XRegistryClientException(e.getLocalizedMessage(),e);
		}

	}

	/**
	 * @return the properties
	 */
	public Properties getProperties() {
		return properties;
	}

	/**
	 * @param properties
	 *            the properties to set
	 */
	public void setProperties(Properties properties) {
		this.properties = properties;
	}

	/**
	 * @return the xRegistryUrl
	 */
	public String getXRegistryUrl() {
		return xRegistryUrl;
	}

	/**
	 * @param xRegistryUrl
	 *            the xRegistryUrl to set
	 */
	public void setXRegistryUrl(String xRegistryUrl) {
		this.xRegistryUrl = xRegistryUrl;
	}

	/**
	 * Returns the gssCredential.
	 * 
	 * @return The gssCredential
	 */
	public GSSCredential getGssCredential() {
		return this.gssCredential;
	}

	/**
	 * Sets gssCredential.
	 * 
	 * @param gssCredential
	 *            The gssCredential to set.
	 */
	public void setGssCredential(GSSCredential gssCredential) {
		this.gssCredential = gssCredential;
	}

	public void setRegistryClient(XRegistryClient registryClient) {
		this.registryClient = registryClient;
	}

	public XRegistryClient getRegistryClient() {
		return registryClient;
	}

	/**
	 * @return the credentials
	 */
	public MyProxyCredentials getCredentials() {
		return credentials;
	}

	/**
	 * @param credentials
	 *            the credentials to set
	 */
	public void setCredentials(MyProxyCredentials credentials) {
		this.credentials = credentials;
	}

	public void setUserDN(String userDN) {
		this.userDN = userDN;
	}

	public String getUserDN() {
		return userDN;
	}
}
